"""Check that the GitHub workflow has a timeout."""

import argparse
import sys
from pathlib import Path

import yaml


def main() -> None:
    """Check that the GitHub workflow has a timeout."""
    parser = argparse.ArgumentParser(description="""Check that the GitHub workflow has a timeout.""")
    parser.add_argument("files", nargs=argparse.REMAINDER, help="The files to check")
    args = parser.parse_args()

    success = True
    files = [Path(filename) for filename in args.files]
    if not files:
        files = [
            *Path(".").glob("github/workflows/*.yaml"),  # noqa: PTH201
            *Path().glob("github/workflows/*.yml"),
        ]
    for filename in files:
        with filename.open(encoding="utf-8") as open_file:
            workflow = yaml.load(open_file, Loader=yaml.SafeLoader)

        for name, job in workflow.get("jobs").items():
            if job.get("timeout-minutes") is None:
                print(f"The workflow '{filename}', job '{name}' has no timeout")
                success = False

    if not success:
        sys.exit(1)


if __name__ == "__main__":
    main()



"""Run a command in files folder."""

import argparse
import subprocess  # nosec
import sys
from pathlib import Path


def main() -> None:
    """Run a command in files folder."""
    parser = argparse.ArgumentParser(
        description="""Run a command in files folder.

    Example:
    pre-commit-run-in-dir --pass-filename --cmd cmd arg1 -aarg2 -a-arg -a--arg --files file1 dir/file2

    will be transform in:
    cmd arg1 arg2 -arg --arg  file1
    (cd dir && cmd arg1 arg2 -arg --arg file2
    """,
    )
    parser.add_argument("--fail-fast", action="store_true", help="Fail on the first error")
    parser.add_argument("--pass-filename", action="store_true", help="Pass the filename to the command")
    parser.add_argument("--check", nargs="+", help="The check command")
    parser.add_argument("--cmd", nargs="+", help="The command", required=True)
    parser.add_argument("-a", "--arg", "--args", nargs="+", help="The args", default=[])
    parser.add_argument("--files", nargs="+", help="The files", required=True)
    args = parser.parse_args()

    command = [*args.cmd, *args.arg]
    success = True
    for file_path in [Path(filename) for filename in args.files]:
        file_path = Path.cwd() / file_path  # noqa: PLW2901
        check_success = True
        if args.check:
            proc = subprocess.run(  # pylint: disable=subprocess-run-check # noqa: S603
                args.check,
                cwd=file_path.parent,
                check=False,
            )
            if proc.returncode != 0:
                if args.fail_fast:
                    sys.exit(proc.returncode)
                check_success = False
        else:
            check_success = False
        if not check_success:
            proc = subprocess.run(  # pylint: disable=subprocess-run-check # noqa: S603
                [*command, file_path.name] if args.pass_filename else command,
                cwd=file_path.parent,
                check=False,
            )
            if proc.returncode != 0:
                if args.fail_fast:
                    sys.exit(proc.returncode)
                success = False
    if not success:
        sys.exit(1)


if __name__ == "__main__":
    main()



# Copyright (c) 2022-2025, Stéphane Brunner
"""Update the copyright header of the files."""

import argparse
import datetime
import re
import subprocess  # nosec
import sys
from datetime import timezone
from pathlib import Path
from typing import TYPE_CHECKING

import yaml

if TYPE_CHECKING:
    StrPattern = re.Pattern[str]
else:
    StrPattern = re.Pattern

CURRENT_YEAR = str(datetime.datetime.now(timezone.utc).year)


def main() -> None:
    """Update the copyright header of the files."""
    args_parser = argparse.ArgumentParser("Update the copyright header of the files")
    args_parser.add_argument("--config", help="The configuration file", default=".github/copyright.yaml")
    args_parser.add_argument("--required", action="store_true", help="The copyright is required")
    args_parser.add_argument("--verbose", action="store_true", help="Verbose mode")
    args_parser.add_argument("files", nargs=argparse.REMAINDER, help="The files to update")
    args = args_parser.parse_args()

    config = {}
    config_path = Path(args.config)
    if config_path.exists():
        with config_path.open(encoding="utf-8") as config_file:
            config = yaml.load(config_file, Loader=yaml.SafeLoader)

    one_date_re = re.compile(config.get("one_date_re", r"\bCopyright \(c\) (?P<year>[0-9]{4})\b"))
    two_date_re = re.compile(
        config.get("two_date_re", r"\bCopyright \(c\) (?P<from>[0-9]{4})-(?P<to>[0-9]{4})\b"),
    )
    one_date_format = config.get("one_date_format", "Copyright (c) {year}")
    two_date_format = config.get("two_date_format", "Copyright (c) {from}-{to}")
    year_re = re.compile(r"^(?P<year>[0-9]{4})-")
    license_file = config.get("license_file", "LICENSE")

    success = True
    no_git_log = False
    for file_name in args.files:
        try:
            status_str = subprocess.run(  # noqa: S603,B607,RUF100
                ["git", "status", "--porcelain", "--", file_name],  # noqa: S607
                check=True,
                encoding="utf-8",
                stdout=subprocess.PIPE,
            ).stdout
            if status_str:
                used_year = CURRENT_YEAR
                if args.verbose:
                    print(f"File '{file_name}' is not committed.")
            else:
                if file_name == license_file:
                    date_str = subprocess.run(  # noqa: S603,B607,RUF100
                        ["git", "log", "--pretty=format:%ci", "-1"],  # noqa: S607
                        check=True,
                        encoding="utf-8",
                        stdout=subprocess.PIPE,
                    ).stdout
                else:
                    date_str = subprocess.run(  # noqa: S603,B607,RUF100
                        ["git", "log", "--follow", "--pretty=format:%ci", "--", file_name],  # noqa: S607
                        check=True,
                        encoding="utf-8",
                        stdout=subprocess.PIPE,
                    ).stdout
                if not date_str:
                    if args.verbose:
                        print(f"No log found with git on '{file_name}'.")
                    elif not no_git_log:
                        print(
                            f"No log found with git on '{file_name}' (the next messages will be hidden).",
                        )
                        no_git_log = True
                    used_year = CURRENT_YEAR
                else:
                    if args.verbose:
                        print(f"File '{file_name}' was committed on '{date_str}'.")
                    used_year_match = year_re.search(date_str)
                    assert used_year_match is not None  # nosec
                    used_year = used_year_match.group("year")
        except FileNotFoundError:
            if not no_git_log:
                print("No Git found.")
                no_git_log = True
            used_year = CURRENT_YEAR
        except subprocess.CalledProcessError as error:
            print(f"Error with Git on '{file_name}' ({error!s}).")
            used_year = CURRENT_YEAR

        with Path(file_name).open(encoding="utf-8") as file_obj:
            content = file_obj.read()
            file_success, content = update_file(
                content,
                used_year,
                one_date_re,
                two_date_re,
                one_date_format,
                two_date_format,
                file_name,
                args.required,
                args.verbose,
            )
        if not file_success:
            success = False
            with Path(file_name).open("w", encoding="utf-8") as file_obj:
                file_obj.write(content)
            if args.verbose:
                print(f"Copyright updated in '{file_name}'.")

    if not success:
        sys.exit(1)


def update_file(
    content: str,
    last_year: str,
    one_date_re: StrPattern,
    two_date_re: StrPattern,
    one_date_format: str,
    two_date_format: str,
    filename: str = "<unknown>",
    required: bool = False,
    verbose: bool = False,
    current_year: str = CURRENT_YEAR,
) -> tuple[bool, str]:
    """Update the copyright header of the file content."""
    two_date_match = two_date_re.search(content)
    if two_date_match:
        if two_date_match.group("from") == two_date_match.group("to"):
            if two_date_match.group("from") == current_year:
                return False, two_date_re.sub(one_date_format.format(year=current_year), content)
            return (
                False,
                two_date_re.sub(
                    two_date_format.format(**{"from": two_date_match.group("from"), "to": current_year}),
                    content,
                ),
            )

        if two_date_match.group("to") == last_year:
            return True, content

        return False, two_date_re.sub(
            two_date_format.format(**{"from": two_date_match.group("from"), "to": current_year}),
            content,
        )

    one_date_match = one_date_re.search(content)
    if one_date_match:
        copyright_year = one_date_match.group("year")

        if copyright_year == last_year:
            return True, content

        return False, one_date_re.sub(
            two_date_format.format(**{"from": copyright_year, "to": current_year}),
            content,
        )

    if required or verbose:
        print(f"No copyright found on '{filename}'.")
    return not required, content


if __name__ == "__main__":
    main()



# Copyright (c) 2025, Stéphane Brunner

import argparse
import io
import re
from pathlib import Path
from typing import Any, AnyStr, Optional, Union

import multi_repo_automation as mra
import ruamel.yaml
import tomlkit


def _order_keys(
    data: dict[str, Any],
    first_keys: list[str],
    last_keys: Optional[list[str]] = None,
) -> ruamel.yaml.comments.CommentedMap:
    if last_keys is None:
        last_keys = []

    # use order: <first_keys>, <other>, <last_keys>
    new_data = [(key, data[key]) for key in first_keys if key in data]

    new_data += [e for e in data.items() if e[0] not in [*first_keys, *last_keys]]

    for key in last_keys:
        if key in data:
            new_data.append((key, data[key]))

    return ruamel.yaml.comments.CommentedMap(new_data)


def _order_sub_keys(
    data: Union[dict[str, dict[str, Any]], list[dict[str, AnyStr]]],
    first_keys: list[str],
    last_keys: Optional[list[str]] = None,
) -> None:
    items = data.items() if isinstance(data, dict) else enumerate(data)
    for key, value in items:
        # copy the last comment
        comment = None
        if hasattr(value, "ca"):
            last_value = list(value.values())[-1]
            if isinstance(last_value, dict) and hasattr(last_value, "ca"):
                last_last_key = list(last_value.keys())[-1]
                comment = last_value.ca.items.get(last_last_key)
                if comment is not None:
                    del last_value.ca.items[last_last_key]
            else:
                comment = value.ca.items.get(list(value.keys())[-1])  # type: ignore[union-attr]
        data[key] = _order_keys(value, first_keys, last_keys)  # type: ignore[index]
        if comment is not None:
            last_key = list(data[key].keys())[-1]  # type: ignore[index]
            if isinstance(data[key][last_key], ruamel.yaml.comments.CommentedMap):  # type: ignore[index]
                last_last_key = list(data[key][last_key].keys())[-1]  # type: ignore[union-attr,index]
                data[key][last_key].ca.items[last_last_key] = comment  # type: ignore[union-attr,index]
            else:
                data[key].ca.items[last_key] = comment  # type: ignore[union-attr,index]


def _canonicalize_workflow(workflow: mra.EditYAML) -> None:
    workflow.data = _order_keys(workflow.data, ["name", "on", "permissions", "env"], ["jobs"])

    # Add space after simple key
    for key in ["name"]:
        workflow.data.ca.items[key] = [
            None,
            None,
            ruamel.yaml.CommentToken("\n\n", ruamel.yaml.error.CommentMark(0), None),
            None,
        ]

    # add space after complex keys
    for key in ["on", "permissions", "env", "jobs"]:
        workflow.data.ca.items[key] = [
            None,
            [ruamel.yaml.CommentToken("\n", ruamel.yaml.error.CommentMark(0), None)],
            None,
            None,
        ]

    for name, job in workflow["jobs"].items():
        job = _order_keys(  # noqa: PLW2901
            job,
            ["name", "runs-on", "timeout-minutes", "if", "concurrency", "needs"],
            ["strategy", "env", "steps"],
        )
        workflow["jobs"][name] = job

        for key in reversed(["name", "runs-on", "timeout-minutes", "if", "concurrency", "needs"]):
            if key in job:
                job.ca.items[key] = [
                    None,
                    None,
                    ruamel.yaml.CommentToken("\n\n", ruamel.yaml.error.CommentMark(0), None),
                    None,
                ]
                break

        for key in reversed(["strategy", "env"]):
            if key in job:
                job.ca.items[key] = [
                    None,
                    None,
                    ruamel.yaml.CommentToken("\n\n", ruamel.yaml.error.CommentMark(0), None),
                    None,
                ]

    for job in workflow["jobs"].values():
        if "steps" in job:
            _order_sub_keys(job["steps"], ["name"], ["uses", "with", "run", "env", "if"])


def _canonicalize_prospector(prospector: mra.EditYAML) -> None:
    prospector.data = _order_keys(
        prospector.data,
        ["inherit"],
        [
            "pylint",
            "mypy",
            "bandit",
            "ruff",
            "pyflakes",
            "pycodestyle",
            "pydocstyle",
            "mccabe",
            "dodgy",
            "pyroma",
            "vulture",
            "frosted",
        ],
    )


def _split_pipe(exclude: str) -> list[str]:
    result = []
    in_parent = False
    file = ""

    for char in exclude:
        if char == "(":
            in_parent = True
        elif char == ")":
            in_parent = False
        elif char == "|" and not in_parent:
            result.append(file.strip())
            file = ""
        file += char

    result.append(file.strip())
    return result


_CLEAN_ENDLINE_RE = re.compile(r"\s*\n\s*")
_GET_START_END_RE = re.compile(r"([^(]*\()(.*)(\)[^)]*)")


def _canonicalize_pre_commit_exclude(exclude: str) -> Optional[ruamel.yaml.scalarstring.LiteralScalarString]:
    exclude = exclude.strip()
    if not exclude.startswith("(?x)"):
        return None
    exclude = exclude[4:]
    exclude = exclude.strip()
    exclude = _CLEAN_ENDLINE_RE.sub("", exclude)
    get_intro_end_match = _GET_START_END_RE.match(exclude)
    if get_intro_end_match is None:
        return None
    start, exclude, end = get_intro_end_match.groups()

    files = _split_pipe(exclude)

    return ruamel.yaml.scalarstring.LiteralScalarString(
        "\n".join(
            [
                f"(?x){start}",
                *[f"  {file}" for file in files],
                end,
            ],
        ),
    )


def _canonicalize_pre_commit(pre_commit_path: Path) -> None:
    with mra.EditYAML(pre_commit_path, run_pre_commit=False) as pre_commit_config:
        if "exclude" in pre_commit_config:
            exclude = _canonicalize_pre_commit_exclude(pre_commit_config["exclude"])
            if exclude is not None:
                pre_commit_config["exclude"] = exclude

        for repo in pre_commit_config["repos"]:
            for hook in repo["hooks"]:
                if "exclude" in hook:
                    exclude = _canonicalize_pre_commit_exclude(hook["exclude"])
                    if exclude is not None:
                        hook["exclude"] = exclude


def _canonicalize_pyproject(path: Path) -> None:
    with path.open() as f:
        doc = tomlkit.parse(f.read())

    new_doc = tomlkit.document()
    if "tool" not in new_doc:
        tool = doc["tool"]
        if isinstance(tool, tomlkit.items.Table) and "ruff" in tool:
            ruff = tool["ruff"]
            if isinstance(ruff, tomlkit.items.Table):
                new_doc["tool"] = {"ruff": ruff}
                if "lint" in ruff:
                    new_tool = new_doc["tool"]
                    assert isinstance(new_tool, tomlkit.items.Table)
                    new_ruff = new_tool["ruff"]
                    assert isinstance(new_ruff, tomlkit.items.Table)
                    new_ruff["lint"] = ruff["lint"]

    for key, value in doc.items():
        if key not in new_doc and key != "build-system":
            new_doc[key] = value
        elif key == "tool":
            for subkey, subvalue in value.items():
                new_tool = new_doc["tool"]
                assert isinstance(new_tool, tomlkit.items.Table)
                if subkey not in new_tool:
                    new_tool[subkey] = subvalue

    if "build-system" in doc:
        new_doc["build-system"] = doc["build-system"]

    # Remove double end of line
    out = io.StringIO()
    tomlkit.dump(new_doc, out, sort_keys=True)
    new_lines = []
    last_empty = False
    for line in out.getvalue().split("\n"):
        if line == "" and last_empty:
            continue
        if line == "":
            last_empty = True
            new_lines.append(line)
        else:
            last_empty = False
            new_lines.append(line)

    with path.open("w") as f:
        f.write("\n".join(new_lines))


def main() -> None:
    """Update the copyright header of the files."""
    args_parser = argparse.ArgumentParser("Format some files like the GitHub workflow")
    args_parser.add_argument("files", nargs=argparse.REMAINDER, type=Path, help="The files to update")
    args = args_parser.parse_args()

    for file_path in args.files:
        # is subpath of .github/workflows
        if file_path.parts[:3] == (".github", "workflows"):
            print(f"Format {file_path} as a GitHub workflow")
            with mra.EditYAML(file_path, run_pre_commit=False) as e:
                _canonicalize_workflow(e)

        elif (
            file_path.name in (".prospector.yaml", ".prospector.yml", "prospector.yaml", "prospector.yml")
            or file_path.name.startswith(".prospector-")
            or file_path.name.startswith("prospector-")
            or (len(file_path.parts) == 2 and file_path.parts[0].startswith("prospector_"))
        ):
            print(f"Format {file_path} as a Prospector configuration")
            with mra.EditYAML(file_path, run_pre_commit=False) as e:
                _canonicalize_prospector(e)

        elif file_path.name == "pyproject.toml":
            print(f"Format {file_path} as a pyproject.toml")
            _canonicalize_pyproject(file_path)

        elif file_path.name in (".pre-commit-config.yaml", ".pre-commit-config.yml"):
            print(f"Format {file_path} as a pre-commit configuration")
            _canonicalize_pre_commit(file_path)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# Get the Ruff configuration from the prospector.yaml and put it in the pyproject.toml

import argparse
from pathlib import Path

import prospector.profiles.profile
import tomlkit


def main() -> None:
    """Update pyproject.toml with Ruff configuration from prospector.yaml."""
    parser = argparse.ArgumentParser(
        description="Update pyproject.toml with Ruff configuration from prospector.yaml",
    )
    parser.add_argument(
        "prospector_config",
        type=Path,
        nargs="+",
        help="Path to the prospector.yaml file containing the Ruff configuration",
    )

    args = parser.parse_args()

    for prospector_config in args.prospector_config:
        if not prospector_config.is_file():
            print(f"File {prospector_config} does not exist")
            continue

        pyproject_path = prospector_config.parent / "pyproject.toml"
        if not pyproject_path.is_file():
            print(f"File {pyproject_path} does not exist")
            continue

        print(f"Processing {prospector_config} and updating {pyproject_path}")

        profile_path: list[Path] = []

        workdir = Path.cwd()

        profile_path.append(workdir)
        profile_path.append(prospector.profiles.profile.BUILTIN_PROFILE_PATH)

        print(f"Using profile path: {', '.join(str(p) for p in profile_path)}")

        profile = prospector.profiles.profile.ProspectorProfile.load(prospector_config.name, profile_path)

        with pyproject_path.open("r", encoding="utf-8") as pyproject_file:
            pyproject_doc = tomlkit.parse(pyproject_file.read())
        ruff_config = pyproject_doc.setdefault("tool", {}).setdefault("ruff", {})
        lint_config = ruff_config.setdefault("lint", {})

        top_level_properties = {
            "builtins",
            "cache-dir",
            "exclude",
            "extend",
            "extend-exclude",
            "extend-include",
            "fix",
            "fix-only",
            "force-exclude",
            "include",
            "indent-width",
            "line-length",
            "namespace-packages",
            "output-format",
            "per-file-target-version",
            "preview",
            "required-version",
            "respect-gitignore",
            "show-fixes",
            "src",
            "target-version",
            "unsafe-fixes",
        }

        for key, value in list(lint_config.items()):
            if not isinstance(value, tomlkit.items.Table):
                del lint_config[key]

        lint_config["select"] = sorted(profile.ruff.get("enable", []))  # pylint: disable=no-member
        lint_config["ignore"] = sorted(profile.ruff.get("disable", []))  # pylint: disable=no-member

        for option, value in profile.ruff.get("options", {}).items():  # pylint: disable=no-member
            current_config = ruff_config if option in top_level_properties else lint_config
            if isinstance(value, dict):
                # If the value is a dictionary, treat it as a set of boolean flags.
                # Extract the keys with truthy values and store them as a list in the Ruff configuration.
                current_config[option] = [k for k, v in value.items() if v]
            else:
                current_config[option] = value
        with pyproject_path.open("w", encoding="utf-8") as pyproject_file:
            tomlkit.dump(pyproject_doc, pyproject_file)


if __name__ == "__main__":
    main()
